@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclCmdAH.c,v 1.7 1999/07/01 23:21:06 redman Exp $
+ * RCS: @(#) $Id: tclCmdAH.c,v 1.7.4.1 1999/09/22 04:12:45 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -1909,6 +1909,8 @@ Tcl_FormatObjCmd(dummy, interp, objc, objv)
 				 * been seen in the current field. */
     int gotPrecision;		/* Non-zero indicates that a precision has
 				 * been set for the current field. */
+    int gotZero;		/* Non-zero indicates that a zero flag has
+ 				 * been seen in the current field. */
 
     /*
      * This procedure is a bit nasty.  The goal is to use sprintf to
@@ -1937,7 +1939,7 @@ Tcl_FormatObjCmd(dummy, interp, objc, objv)
 	register char *newPtr = newFormat;
 
 	width = precision = noPercent = useShort = 0;
-	gotMinus = gotPrecision = 0;
+	gotZero = gotMinus = gotPrecision = 0;
 	whichValue = PTR_VALUE;
 
 	/*
@@ -2006,6 +2008,13 @@ Tcl_FormatObjCmd(dummy, interp, objc, objv)
 	    if (*format == '-') {
 		gotMinus = 1;
 	    }
+ 	    if (*format == '0') {
+ 		/*
+ 		 * This will be handled by sprintf for numbers, but we
+ 		 * need to do the char/string ones ourselves
+ 		 */
+ 		gotZero = 1;
+ 	    }
 	    *newPtr = *format;
 	    newPtr++;
 	    format++;
@@ -2204,21 +2213,23 @@ Tcl_FormatObjCmd(dummy, interp, objc, objv)
 		}
 		case CHAR_VALUE: {
 		    char *ptr;
+		    char padChar = (gotZero ? '0' : ' ');
 		    ptr = dst;
 		    if (!gotMinus) {
 			for ( ; --width > 0; ptr++) {
-			    *ptr = ' ';
+			    *ptr = padChar;
 			}
 		    }
 		    ptr += Tcl_UniCharToUtf(intValue, ptr);
 		    for ( ; --width > 0; ptr++) {
-			*ptr = ' ';
+			*ptr = padChar;
 		    }
 		    *ptr = '\0';
 		    break;
 		}
 		case STRING_VALUE: {
 		    char *ptr;
+		    char padChar = (gotZero ? '0' : ' ');
 		    int pad;
 
 		    ptr = dst;
@@ -2230,7 +2241,7 @@ Tcl_FormatObjCmd(dummy, interp, objc, objv)
 
 		    if (!gotMinus) {
 			while (pad > 0) {
-			    *ptr++ = ' ';
+			    *ptr++ = padChar;
 			    pad--;
 			}
 		    }
@@ -2241,7 +2252,7 @@ Tcl_FormatObjCmd(dummy, interp, objc, objv)
 			ptr += size;
 		    }
 		    while (pad > 0) {
-			*ptr++ = ' ';
+			*ptr++ = padChar;
 			pad--;
 		    }
 		    *ptr = '\0';
