@@ -61,11 +61,6 @@
 # endif
 #endif
 
-#ifndef __va_copy
-#define __va_copy(tmpArgList, argList) \
-    memcpy ((VOID *) &tmpArgList, (VOID *) &argList, sizeof (tmpArgList));
-#endif
-
 /*
  * The following procedures allow namespaces to be customized to
  * support special name resolution rules for commands/variables.
