@@ -14,7 +14,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclParse.c,v 1.7 1999/05/04 01:32:12 stanton Exp $
+ * RCS: @(#) $Id: tclParse.c,v 1.12 1999/08/12 23:14:42 stanton Exp $
  */
 
 #include "tclInt.h"
@@ -260,6 +260,7 @@ Tcl_ParseCommand(interp, string, numBytes, nested, parsePtr)
     parsePtr->tokensAvailable = NUM_STATIC_TOKENS;
     parsePtr->string = string;
     parsePtr->end = string + numBytes;
+    parsePtr->term = parsePtr->end;
     parsePtr->interp = interp;
     parsePtr->incomplete = 0;
     parsePtr->errorType = TCL_PARSE_SUCCESS;
@@ -372,6 +373,7 @@ Tcl_ParseCommand(interp, string, numBytes, nested, parsePtr)
 	    break;
 	}
 	if ((type & terminators) != 0) {
+	    parsePtr->term = src;
 	    src++;
 	    break;
 	}
@@ -455,6 +457,7 @@ Tcl_ParseCommand(interp, string, numBytes, nested, parsePtr)
 	}
 
 	if ((type & terminators) != 0) {
+	    parsePtr->term = src;
 	    src++;
 	    break;
 	}
@@ -607,7 +610,7 @@ ParseTokens(src, mask, parsePtr)
 		if (nested.tokenPtr != nested.staticTokens) {
 		    ckfree((char *) nested.tokenPtr);
 		}
-		if ((src[-1] == ']') && !nested.incomplete) {
+		if ((*nested.term == ']') && !nested.incomplete) {
 		    break;
 		}
 		if (src == parsePtr->end) {
@@ -1453,7 +1456,19 @@ Tcl_EvalEx(interp, script, numBytes, flags)
 	Tcl_DecrRefCount(objv[i]);
     }
     if (gotParse) {
+	p = parse.commandStart + parse.commandSize;
 	Tcl_FreeParse(&parse);
+	if ((nested != 0) && (p > script) && (p[-1] == ']')) {
+	    /*
+	     * We get here in the special case where the TCL_BRACKET_TERM
+	     * flag was set in the interpreter and we reached a close
+	     * bracket in the script.  Return immediately.
+	     */
+
+	    iPtr->termOffset = (p - 1) - script;
+	} else {
+	    iPtr->termOffset = p - script;
+	}    
     }
     if (objv != staticObjArray) {
 	ckfree((char *) objv);
@@ -1592,6 +1607,7 @@ Tcl_ParseVarName(interp, string, numBytes, parsePtr, append)
     unsigned char c;
     int varIndex, offset;
     Tcl_UniChar ch;
+    unsigned array;
 
     if (numBytes >= 0) {
 	end = string + numBytes;
@@ -1698,12 +1714,17 @@ Tcl_ParseVarName(interp, string, numBytes, parsePtr, append)
 	    }
 	    break;
 	}
+
+	/*
+	 * Support for empty array names here.
+	 */
+	array = ((src != end) && (*src == '('));
 	tokenPtr->size = src - tokenPtr->start;
-	if (tokenPtr->size == 0) {
+	if (tokenPtr->size == 0 && !array) {
 	    goto justADollarSign;
 	}
 	parsePtr->numTokens++;
-	if ((src != end) && (*src == '(')) {
+	if (array) {
 	    /*
 	     * This is a reference to an array element.  Call
 	     * ParseTokens recursively to parse the element name,
