@@ -114,7 +114,7 @@ TCL_DECLARE_MUTEX(threadMutex)
 #undef TCL_STORAGE_CLASS
 #define TCL_STORAGE_CLASS DLLEXPORT
 
-EXTERN int	TclThread_Init _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN int	Thread_Init _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN int	Thread_Init _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN int	Tcl_ThreadObjCmd _ANSI_ARGS_((ClientData clientData,
 	Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
@@ -168,7 +168,9 @@ int
 Thread_Init(interp)
     Tcl_Interp *interp; /* The current Tcl interpreter */
 {
-    Tcl_InitStubs(interp, TCL_VERSION, 0);
+    if (Tcl_InitStubs(interp, "8.1", 0) == NULL) {
+	return TCL_ERROR;
+    }
     Tcl_CreateObjCommand(interp,"testthread", Tcl_ThreadObjCmd, 
 	    (ClientData)NULL ,NULL);
     if (Tcl_PkgProvide(interp, "Thread", "1.0" ) != TCL_OK) {
@@ -431,7 +433,7 @@ NewThread(clientData)
 
     tsdPtr->interp = Tcl_CreateInterp();
     result = Tcl_Init(tsdPtr->interp);
-    result = Threadtest_Init(tsdPtr->interp);
+    result = Thread_Init(tsdPtr->interp);
 
     /*
      * Update the list of threads.
